// BrushDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Impressionist.h"
#include "ImpressionistDoc.h"
#include "BrushDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBrushDialog dialog


void CBrushDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBrushDialog)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBrushDialog, CDialog)
	//{{AFX_MSG_MAP(CBrushDialog)
	ON_CBN_SELCHANGE(IDC_BRUSH, OnSelectBrush)
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBrushDialog message handlers

BOOL CBrushDialog::Create()
{
	return CDialog::Create(CBrushDialog::IDD);
}

CBrushDialog::CBrushDialog( CView* pWnd )
{
	m_pWnd = pWnd;

	m_nSize = 10;
	m_nSizeMin = 1;
	m_nSizeMax = 20;
	m_nSizeTicFreq = 1;
}

void CBrushDialog::OnOK() 
{
	TRACE( "CBrushDialog::OnOK\n" );

	if ( m_pWnd != NULL ) {
		DestroyWindow();
	} else {
		CDialog::OnOK();
	}
}

void CBrushDialog::OnCancel()
{
	TRACE( "CBrushDialog::OnCancel\n" );
	
	if ( m_pWnd != NULL ) {
		DestroyWindow();
	} else {
		CDialog::OnOK();
	}
}

BOOL CBrushDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CImpressionistDoc* pDoc = (CImpressionistDoc*) m_pWnd->GetDocument();

	CComboBox* pCB = (CComboBox*) GetDlgItem( IDC_BRUSH );
	pCB->ResetContent();
	for ( int i = 0; i < CImpBrush::c_nBrushCount; ++i )
	{
		pCB->AddString( CImpBrush::c_pBrushes[i]->BrushName() );
		if ( pDoc->m_pCurrentBrush == CImpBrush::c_pBrushes[i] )
			pCB->SetCurSel( i );
	}
	
	CString str;
	CSliderCtrl* pSlide;

	pSlide = (CSliderCtrl*) GetDlgItem( IDC_BRUSH_SIZE );
	TRACE( "setting slider to %d (%d-%d)\n", m_nSize, m_nSizeMin, m_nSizeMax );
	pSlide->SetRange ( m_nSizeMin, m_nSizeMax );
	pSlide->SetTicFreq( m_nSizeTicFreq );
	pSlide->SetPos( m_nSize );
	str.Format( "%d", m_nSize );
	SetDlgItemText( IDC_STATIC_SIZE, str );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CBrushDialog::OnSelectBrush() 
{
	CImpressionistDoc* pDoc = (CImpressionistDoc*) m_pWnd->GetDocument();
	CComboBox* pCB = (CComboBox*) GetDlgItem( IDC_BRUSH );
	pDoc->m_pCurrentBrush = CImpBrush::c_pBrushes[ pCB->GetCurSel() ];
	pDoc->m_pCurrentBrush->BrushSelect( this );
}

int CBrushDialog::GetSize( void )
{
	return m_nSize;
}

void CBrushDialog::SetSize( int v )
{
	if ( v < m_nSizeMin ) v = m_nSizeMin;
	else if ( v > m_nSizeMax ) v = m_nSizeMax;

	m_nSize = v;

	if ( GetSafeHwnd() )
	{
		CSliderCtrl* pSlide = (CSliderCtrl*) GetDlgItem( IDC_BRUSH_SIZE );
		pSlide->SetPos( m_nSize );
		CString str;
		str.Format( "%d", m_nSize );
		SetDlgItemText( IDC_STATIC_SIZE, str );
	}
}

void CBrushDialog::SetSizeRange( int min, int max, int tics )
{
	if ( min > max ) return;

	TRACE( "tics starts at %d\n", tics );
	TRACE( "min is %d, max is %d\n", min, max );

	if ( tics == -1 )
	{
		if ( max - min <= 30 ) tics = 1;
		else tics = (max - min + 1) / 10;
	}

	TRACE( "tics is %d\n", tics );

	m_nSizeTicFreq = tics;
	m_nSizeMin = min;
	m_nSizeMax = max;
	if ( m_nSize < min ) m_nSize = min;
	else if ( m_nSize > max ) m_nSize = max;

	if ( GetSafeHwnd() )
	{
		CSliderCtrl* pSlide = (CSliderCtrl*) GetDlgItem( IDC_BRUSH_SIZE );
		pSlide->SetRange( min, max );
		pSlide->SetTicFreq( tics );
		pSlide->SetPos( m_nSize );
		CString str;
		str.Format( "%d", m_nSize );
		SetDlgItemText( IDC_STATIC_SIZE, str );
	}
}

void CBrushDialog::GetSizeRange( int* min, int *max, int *tics )
{
	if ( min ) *min = m_nSizeMin;
	if ( max ) *max = m_nSizeMax;
	if ( tics ) *tics = m_nSizeTicFreq;
}

void CBrushDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CSliderCtrl* pSlide = (CSliderCtrl*) pScrollBar;
	CString str;

	switch( pScrollBar->GetDlgCtrlID() ) {

	case IDC_BRUSH_SIZE:
		m_nSize = pSlide->GetPos();
		str.Format( "%d", m_nSize );
		SetDlgItemText( IDC_STATIC_SIZE, str );
		break;

	}
}
