// ImpressionistDoc.cpp : implementation of the CImpressionistDoc class
//

#include "stdafx.h"
#include "Impressionist.h"

#include "ImpressionistDoc.h"
#include "DotBrush.h"
#include "BrushDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CImpressionistDoc

IMPLEMENT_DYNCREATE(CImpressionistDoc, CDocument)

BEGIN_MESSAGE_MAP(CImpressionistDoc, CDocument)
	//{{AFX_MSG_MAP(CImpressionistDoc)
	ON_COMMAND(IDM_SELECT, OnImageSelect)
	ON_COMMAND(IDM_BRUSH, OnBrush)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CImpressionistDoc construction/destruction

CImpressionistDoc::CImpressionistDoc() :
	m_pCurrentBrush(NULL), m_pPaintView(NULL), m_pBrushDlg(NULL)
{
}

CImpressionistDoc::~CImpressionistDoc()
{
	delete m_pBrushDlg;
}

BOOL CImpressionistDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	m_nWidth = -1;
	m_ucBitmap = NULL;
	m_ucPainting = NULL;

	// create one instance of each brush
	CImpBrush::c_nBrushCount = 1;
	CImpBrush::c_pBrushes = new CImpBrush* [CImpBrush::c_nBrushCount];
	CImpBrush::c_pBrushes[0] = new CDotBrush( this, "Points" );

	// make one of the brushes current
	m_pCurrentBrush = CImpBrush::c_pBrushes[0];

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CImpressionistDoc serialization

void CImpressionistDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CImpressionistDoc diagnostics

#ifdef _DEBUG
void CImpressionistDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CImpressionistDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CImpressionistDoc commands

void CImpressionistDoc::OnCloseDocument() 
{
	if ( m_ucBitmap ) delete [] m_ucBitmap;
	if ( m_ucPainting ) delete [] m_ucPainting;

	CDocument::OnCloseDocument();
}

void CImpressionistDoc::OnImageSelect() 
{
	CFileDialog dlg( TRUE, "bmp", "*.bmp" );
	if ( dlg.DoModal() == IDOK ) {
		CFile file;
		DWORD pos;
		unsigned int foo;

		VERIFY( file.Open( dlg.GetPathName(), CFile::modeRead ) );

		BITMAPFILEHEADER bmfh;
		BITMAPINFOHEADER bmih;
		
		file.Read( &bmfh, sizeof(BITMAPFILEHEADER) );
		pos = bmfh.bfOffBits;
		file.Read( &bmih, sizeof(BITMAPINFOHEADER) );

		//TRACE( "bitmap is %d x %d\n", bmih.biWidth, bmih.biHeight );
		//TRACE( "offset is %d\n", pos );
		//TRACE( "tag is %x\n", bmfh.bfType );

		if ( bmih.biBitCount != 24 ) {
			MessageBox( NULL, "Only 24-bit BMPs are supported.", NULL, MB_OK | MB_ICONEXCLAMATION );
			return;
		}
		if ( bmih.biCompression != BI_RGB ) {
			MessageBox( NULL, "Compressed BMPs are not supported.", NULL, MB_OK | MB_ICONEXCLAMATION );
			return;
		}

		file.Seek( pos, CFile::begin );

		int bytes, pad;
		bytes = bmih.biWidth * 3;
		pad = 0;
		if ( bytes % 4 != 0 )
		{
			pad = 4 - (bytes % 4);
			bytes += pad;
		}
		bytes *= bmih.biHeight;

		m_ucBitmap = new unsigned char [bytes];

		foo = file.Read( m_ucBitmap, bytes );		
		//TRACE( "wanted %d bytes, got %d\n", bytes, foo );
			
		file.Close();

		m_nPaintWidth = m_nWidth = bmih.biWidth;
		m_nPaintHeight = m_nHeight = bmih.biHeight;
	
		int i, j;
		j = 0;
		unsigned char temp;
		unsigned char* in;
		unsigned char* out;

		in = m_ucBitmap;
		out = m_ucBitmap;

		for ( j = 0; j < m_nHeight; ++j )
		{
			for ( i = 0; i < m_nWidth; ++i )
			{
				out[1] = in[1];
				temp = in[2];
				out[2] = in[0];
				out[0] = temp;

				in += 3;
				out += 3;
			}
			in += pad;
		}

		UpdateAllViews( NULL, UPDATE_HINT_NEWORIG );

	} 

}

GLubyte* CImpressionistDoc::GetOriginalPixel( int x, int y )
{
	if ( x < 0 ) x = 0;
	else if ( x >= m_nWidth ) x = m_nWidth-1;
	if ( y < 0 ) y = 0;
	else if ( y >= m_nHeight ) y = m_nHeight-1;

	return (GLubyte*)(m_ucBitmap + 3 * (y*m_nWidth + x));
}

GLubyte* CImpressionistDoc::GetOriginalPixel( const CPoint p )
{
	return GetOriginalPixel( p.x, p.y );
}



void CImpressionistDoc::OnBrush() 
{
	if ( m_pBrushDlg->GetSafeHwnd() == 0 ) {
		m_pBrushDlg->Create();
	}
	
}

void CImpressionistDoc::AllocateDialog()
{
	if ( m_pBrushDlg == NULL )
		m_pBrushDlg = new CBrushDialog( m_pPaintView );

}

void CImpressionistDoc::OnFileSave() 
{
	if ( m_ucPainting == NULL )
		return;

	CFileDialog dlg( FALSE, "BMP", NULL, OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY, "BMP files (*.bmp)|*.bmp|All Files (*.*)|*.*||", NULL );

	if ( dlg.DoModal() == IDOK )
	{
		BITMAPFILEHEADER bmfh;
		BITMAPINFOHEADER bmih;

		int bytes, pad;
		bytes = m_nPaintWidth * 3;
		pad = (bytes%4) ? 4-(bytes%4) : 0;
		bytes += pad;
		bytes *= m_nPaintHeight;

		bmfh.bfType = 0x4d42;    // "BM"
		bmfh.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + bytes;
		bmfh.bfReserved1 = 0;
		bmfh.bfReserved2 = 0;
		bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

		bmih.biSize = sizeof(BITMAPINFOHEADER);
		bmih.biWidth = m_nPaintWidth;
		bmih.biHeight = m_nPaintHeight;
		bmih.biPlanes = 1;
		bmih.biBitCount = 24;
		bmih.biCompression = BI_RGB;
		bmih.biSizeImage = 0;
		bmih.biXPelsPerMeter = (int)(100 / 2.54 * 72);
		bmih.biYPelsPerMeter = (int)(100 / 2.54 * 72);
		bmih.biClrUsed = 0;
		bmih.biClrImportant = 0;

		CFile file( dlg.GetPathName(), CFile::modeWrite | CFile::modeCreate );
		file.Write( &bmfh, sizeof(BITMAPFILEHEADER) );
		file.Write( &bmih, sizeof(BITMAPINFOHEADER) );

		bytes /= m_nPaintHeight;
		unsigned char* scanline = new unsigned char [bytes];
		for ( int j = 0; j < m_nPaintHeight; ++j )
		{
			memcpy( scanline, m_ucPainting + j*3*m_nPaintWidth, bytes );
			for ( int i = 0; i < m_nPaintWidth; ++i )
			{
				unsigned char temp = scanline[i*3];
				scanline[i*3] = scanline[i*3+2];
				scanline[i*3+2] = temp;
			}
			file.Write( scanline, bytes );
		}
		delete [] scanline;
		file.Close();

	}
}
